@@ -18,13 +18,6 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-import sys
-
-# TODO: #6568 Remove this hack that makes dlopen() not crash.
-if hasattr(sys, "getdlopenflags") and hasattr(sys, "setdlopenflags"):
-  import ctypes
-  sys.setdlopenflags(sys.getdlopenflags() | ctypes.RTLD_GLOBAL)
-
 import numpy as np
 
 from tensorflow.contrib.layers.python.layers import optimizers as optimizers_lib
@@ -108,6 +101,14 @@ class OptimizersTest(test.TestCase):
             optimizers_lib.optimize_loss(
                 loss, global_step, learning_rate=0.1, optimizer=optimizer)
 
+  def testBadSummaries(self):
+    with ops.Graph().as_default() as g, self.test_session(graph=g):
+      _, _, loss, global_step = _setup_model()
+      with self.assertRaises(ValueError):
+        optimizers_lib.optimize_loss(
+            loss, global_step, learning_rate=0.1, optimizer="SGD",
+            summaries=["loss", "bad_summary"])
+
   def testInvalidLoss(self):
     with ops.Graph().as_default() as g, self.test_session(graph=g):
       _, _, _, global_step = _setup_model()
@@ -124,7 +125,7 @@ class OptimizersTest(test.TestCase):
       var = variable_scope.get_variable(
           "test", [], initializer=init_ops.constant_initializer(10))
       loss = math_ops.abs(var * x)
-      with self.assertRaises(TypeError):
+      with self.assertRaises(AttributeError):
         optimizers_lib.optimize_loss(
             loss,
             global_step=constant_op.constant(
@@ -357,6 +358,30 @@ class OptimizersTest(test.TestCase):
         self.assertEqual(20, update_var.eval())
         self.assertEqual(1, global_step.eval())
 
+  def testUpdateOpNoIncrementGlobalStep(self):
+    optimizers = [
+        "SGD", gradient_descent.GradientDescentOptimizer,
+        gradient_descent.GradientDescentOptimizer(learning_rate=0.1)
+    ]
+    for optimizer in optimizers:
+      with ops.Graph().as_default() as g, self.test_session(graph=g) as session:
+        x, var, loss, global_step = _setup_model()
+        update_var = variable_scope.get_variable(
+            "update", [], initializer=init_ops.constant_initializer(10))
+        update_op = state_ops.assign(update_var, 20)
+        train = optimizers_lib.optimize_loss(
+            loss,
+            global_step,
+            learning_rate=0.1,
+            optimizer=optimizer,
+            update_ops=[update_op],
+            increment_global_step=False)
+        variables.global_variables_initializer().run()
+        session.run(train, feed_dict={x: 5})
+        self.assertEqual(9.5, var.eval())
+        self.assertEqual(20, update_var.eval())
+        self.assertEqual(0, global_step.eval())
+
   def testUpdateOpWithNoOpDecay(self):
     optimizers = [
         "SGD", gradient_descent.GradientDescentOptimizer,
