@@ -19,28 +19,29 @@ from __future__ import division
 from __future__ import print_function
 
 import math
-import sys
 import time
 
-# TODO: #6568 Remove this hack that makes dlopen() not crash.
-if hasattr(sys, 'getdlopenflags') and hasattr(sys, 'setdlopenflags'):
-  import ctypes
-  sys.setdlopenflags(sys.getdlopenflags() | ctypes.RTLD_GLOBAL)
-
 import numpy as np
 from sklearn.cluster import KMeans as SklearnKMeans
 
-from tensorflow.contrib import factorization
+# pylint: disable=g-import-not-at-top
+from tensorflow.contrib.learn.python import learn
 from tensorflow.contrib.learn.python.learn.estimators import kmeans as kmeans_lib
 from tensorflow.contrib.learn.python.learn.estimators import run_config
 from tensorflow.python.framework import constant_op
 from tensorflow.python.framework import dtypes
+from tensorflow.python.framework import ops
 from tensorflow.python.ops import array_ops
+from tensorflow.python.ops import control_flow_ops
+from tensorflow.python.ops import data_flow_ops
+from tensorflow.python.ops import math_ops
+from tensorflow.python.ops import random_ops
 from tensorflow.python.ops import random_ops
 from tensorflow.python.platform import benchmark
 from tensorflow.python.platform import flags
 from tensorflow.python.platform import test
 from tensorflow.python.training import input as input_lib
+from tensorflow.python.training import queue_runner
 
 FLAGS = flags.FLAGS
 
@@ -69,25 +70,43 @@ def make_random_points(centers, num_points, max_offset=20):
 
 class KMeansTestBase(test.TestCase):
 
-  def input_fn(self, batch_size=None, points=None, num_epochs=None):
+  def input_fn(self, batch_size=None, points=None, randomize=None,
+               num_epochs=None):
     """Returns an input_fn that randomly selects batches from given points."""
     batch_size = batch_size or self.batch_size
     points = points if points is not None else self.points
     num_points = points.shape[0]
-
+    if randomize is None:
+      randomize = (self.use_mini_batch and
+                   self.mini_batch_steps_per_iteration <= 1)
     def _fn():
       x = constant_op.constant(points)
       if batch_size == num_points:
         return input_lib.limit_epochs(x, num_epochs=num_epochs), None
-      indices = random_ops.random_uniform(
-          constant_op.constant([batch_size]),
-          minval=0,
-          maxval=num_points - 1,
-          dtype=dtypes.int32,
-          seed=10)
+      if randomize:
+        indices = random_ops.random_uniform(
+            constant_op.constant([batch_size]),
+            minval=0, maxval=num_points-1,
+            dtype=dtypes.int32,
+            seed=10)
+      else:
+        # We need to cycle through the indices sequentially. We create a queue
+        # to maintain the list of indices.
+        q = data_flow_ops.FIFOQueue(self.num_points, dtypes.int32, ())
+        # Conditionally initialize the Queue.
+        def _init_q():
+          with ops.control_dependencies([q.enqueue_many(
+              math_ops.range(self.num_points))]):
+            return control_flow_ops.no_op()
+        init_q = control_flow_ops.cond(q.size() <= 0,
+                                       _init_q,
+                                       control_flow_ops.no_op)
+        with ops.control_dependencies([init_q]):
+          offsets = q.dequeue_many(self.batch_size)
+          with ops.control_dependencies([q.enqueue_many(offsets)]):
+            indices = array_ops.identity(offsets)
       batch = array_ops.gather(x, indices)
       return (input_lib.limit_epochs(batch, num_epochs=num_epochs), None)
-
     return _fn
 
   @staticmethod
@@ -102,6 +121,10 @@ class KMeansTestBase(test.TestCase):
   def use_mini_batch(self):
     return False
 
+  @property
+  def mini_batch_steps_per_iteration(self):
+    return 1
+
 
 class KMeansTest(KMeansTestBase):
 
@@ -109,7 +132,7 @@ class KMeansTest(KMeansTestBase):
     np.random.seed(3)
     self.num_centers = 5
     self.num_dims = 2
-    self.num_points = 10000
+    self.num_points = 1000
     self.true_centers = make_random_centers(self.num_centers, self.num_dims)
     self.points, _, self.scores = make_random_points(self.true_centers,
                                                      self.num_points)
@@ -118,10 +141,11 @@ class KMeansTest(KMeansTestBase):
   def _kmeans(self, relative_tolerance=None):
     return kmeans_lib.KMeansClustering(
         self.num_centers,
-        initial_clusters=factorization.RANDOM_INIT,
+        initial_clusters=kmeans_lib.KMeansClustering.KMEANS_PLUS_PLUS_INIT,
+        distance_metric=kmeans_lib.KMeansClustering.SQUARED_EUCLIDEAN_DISTANCE,
         use_mini_batch=self.use_mini_batch,
-        config=self.config(14),
-        random_seed=10,
+        mini_batch_steps_per_iteration=self.mini_batch_steps_per_iteration,
+        random_seed=24,
         relative_tolerance=relative_tolerance)
 
   def test_clusters(self):
@@ -144,12 +168,15 @@ class KMeansTest(KMeansTestBase):
 
   def test_monitor(self):
     if self.use_mini_batch:
+      # We don't test for use_mini_batch case since the loss value can be noisy.
       return
     kmeans = kmeans_lib.KMeansClustering(
         self.num_centers,
-        initial_clusters=factorization.RANDOM_INIT,
+        initial_clusters=kmeans_lib.KMeansClustering.KMEANS_PLUS_PLUS_INIT,
+        distance_metric=kmeans_lib.KMeansClustering.SQUARED_EUCLIDEAN_DISTANCE,
         use_mini_batch=self.use_mini_batch,
-        config=run_config.RunConfig(tf_random_seed=14),
+        mini_batch_steps_per_iteration=self.mini_batch_steps_per_iteration,
+        config=learn.RunConfig(tf_random_seed=14),
         random_seed=12,
         relative_tolerance=1e-4)
 
@@ -159,11 +186,13 @@ class KMeansTest(KMeansTestBase):
         steps=None)
     score = kmeans.score(
         input_fn=self.input_fn(batch_size=self.num_points), steps=1)
-    self.assertNear(self.true_score, score, self.true_score * 0.005)
+    self.assertNear(self.true_score, score, self.true_score * 0.01)
 
   def test_infer(self):
-    kmeans = self._kmeans(relative_tolerance=1e-4)
-    kmeans.fit(input_fn=self.input_fn())
+    kmeans = self._kmeans()
+    # Make a call to fit to initialize the cluster centers.
+    max_steps = 1
+    kmeans.fit(input_fn=self.input_fn(), max_steps=max_steps)
     clusters = kmeans.clusters()
 
     # Make a small test set
@@ -194,8 +223,11 @@ class KMeansTest(KMeansTestBase):
     points = np.array([[2.0, 3.0], [1.6, 8.2]], dtype=np.float32)
 
     with self.assertRaisesOpError('less'):
-      kmeans = kmeans_lib.KMeansClustering(
-          num_clusters=3, initial_clusters=factorization.RANDOM_INIT)
+      kmeans = learn.KMeansClustering(
+          num_clusters=3,
+          use_mini_batch=self.use_mini_batch,
+          mini_batch_steps_per_iteration=self.mini_batch_steps_per_iteration,
+          initial_clusters=kmeans_lib.KMeansClustering.RANDOM_INIT)
       kmeans.fit(input_fn=lambda: (constant_op.constant(points), None),
                  steps=10)
 
@@ -204,13 +236,42 @@ class KMeansTest(KMeansTestBase):
     points = np.array([[2.0, 3.0], [1.6, 8.2]], dtype=np.float32)
 
     with self.assertRaisesOpError(AssertionError):
-      kmeans = kmeans_lib.KMeansClustering(
-          num_clusters=3, initial_clusters=factorization.KMEANS_PLUS_PLUS_INIT)
+      kmeans = learn.KMeansClustering(
+          num_clusters=3,
+          use_mini_batch=self.use_mini_batch,
+          mini_batch_steps_per_iteration=self.mini_batch_steps_per_iteration,
+          initial_clusters=kmeans_lib.KMeansClustering.KMEANS_PLUS_PLUS_INIT)
       kmeans.fit(input_fn=lambda: (constant_op.constant(points), None),
                  steps=10)
 
 
-class KMeansTestCosineDistance(KMeansTestBase):
+class MiniBatchKMeansTest(KMeansTest):
+
+  @property
+  def batch_size(self):
+    return 50
+
+  @property
+  def use_mini_batch(self):
+    return True
+
+
+class FullBatchAsyncKMeansTest(KMeansTest):
+
+  @property
+  def batch_size(self):
+    return 50
+
+  @property
+  def use_mini_batch(self):
+    return True
+
+  @property
+  def mini_batch_steps_per_iteration(self):
+    return self.num_points // self.batch_size
+
+
+class KMeansCosineDistanceTest(KMeansTestBase):
 
   def setUp(self):
     self.points = np.array(
@@ -228,50 +289,52 @@ class KMeansTestCosineDistance(KMeansTestBase):
                     normalize(self.points)[4:, :], axis=0, keepdims=True))[0]
         ],
         dtype=np.float32)
-    self.true_assignments = [0] * 4 + [1] * 4
+    self.true_assignments = np.array([0] * 4 + [1] * 4)
     self.true_score = len(self.points) - np.tensordot(
         normalize(self.points), self.true_centers[self.true_assignments])
 
     self.num_centers = 2
     self.kmeans = kmeans_lib.KMeansClustering(
         self.num_centers,
-        initial_clusters=factorization.RANDOM_INIT,
-        distance_metric=factorization.COSINE_DISTANCE,
+        initial_clusters=kmeans_lib.KMeansClustering.RANDOM_INIT,
+        distance_metric=kmeans_lib.KMeansClustering.COSINE_DISTANCE,
         use_mini_batch=self.use_mini_batch,
+        mini_batch_steps_per_iteration=self.mini_batch_steps_per_iteration,
         config=self.config(3))
 
   def test_fit(self):
-    self.kmeans.fit(input_fn=self.input_fn(), steps=10)
+    max_steps = 10 * self.num_points // self.batch_size
+    self.kmeans.fit(input_fn=self.input_fn(), max_steps=max_steps)
     centers = normalize(self.kmeans.clusters())
-    self.assertAllClose(
-        np.sort(
-            centers, axis=0), np.sort(
-                self.true_centers, axis=0))
+    centers = centers[centers[:, 0].argsort()]
+    true_centers = self.true_centers[self.true_centers[:, 0].argsort()]
+    self.assertAllClose(centers, true_centers, atol=0.04)
 
   def test_transform(self):
     self.kmeans.fit(input_fn=self.input_fn(), steps=10)
     centers = normalize(self.kmeans.clusters())
     true_transform = 1 - cosine_similarity(self.points, centers)
-    transform = self.kmeans.transform(input_fn=self.input_fn())
+    transform = self.kmeans.transform(input_fn=self.input_fn(
+        batch_size=self.num_points))
     self.assertAllClose(transform, true_transform, atol=1e-3)
 
   def test_predict(self):
-    self.kmeans.fit(input_fn=self.input_fn(), steps=30)
-
+    max_steps = 10 * self.num_points // self.batch_size
+    self.kmeans.fit(input_fn=self.input_fn(), max_steps=max_steps)
     centers = normalize(self.kmeans.clusters())
-    self.assertAllClose(
-        np.sort(
-            centers, axis=0), np.sort(
-                self.true_centers, axis=0), atol=1e-2)
 
     assignments = list(self.kmeans.predict_cluster_idx(
-        input_fn=self.input_fn(num_epochs=1)))
+        input_fn=self.input_fn(num_epochs=1, batch_size=self.num_points)))
     self.assertAllClose(
         centers[assignments],
         self.true_centers[self.true_assignments],
         atol=1e-2)
 
-    score = self.kmeans.score(input_fn=self.input_fn(), steps=1)
+    centers = centers[centers[:, 0].argsort()]
+    true_centers = self.true_centers[self.true_centers[:, 0].argsort()]
+    self.assertAllClose(centers, true_centers, atol=0.04)
+    score = self.kmeans.score(input_fn=self.input_fn(
+        batch_size=self.num_points), steps=1)
     self.assertAllClose(score, self.true_score, atol=1e-2)
 
   def test_predict_kmeans_plus_plus(self):
@@ -300,9 +363,10 @@ class KMeansTestCosineDistance(KMeansTestBase):
 
     kmeans = kmeans_lib.KMeansClustering(
         3,
-        initial_clusters=factorization.KMEANS_PLUS_PLUS_INIT,
-        distance_metric=factorization.COSINE_DISTANCE,
+        initial_clusters=kmeans_lib.KMeansClustering.KMEANS_PLUS_PLUS_INIT,
+        distance_metric=kmeans_lib.KMeansClustering.COSINE_DISTANCE,
         use_mini_batch=self.use_mini_batch,
+        mini_batch_steps_per_iteration=self.mini_batch_steps_per_iteration,
         config=self.config(3))
     kmeans.fit(input_fn=lambda: (constant_op.constant(points), None), steps=30)
 
@@ -323,17 +387,32 @@ class KMeansTestCosineDistance(KMeansTestBase):
     self.assertAllClose(score, true_score, atol=1e-2)
 
 
-class MiniBatchKMeansTest(KMeansTest):
+class MiniBatchKMeansCosineTest(KMeansCosineDistanceTest):
 
   @property
   def batch_size(self):
-    return 450
+    return 2
 
   @property
   def use_mini_batch(self):
     return True
 
 
+class FullBatchAsyncKMeansCosineTest(KMeansCosineDistanceTest):
+
+  @property
+  def batch_size(self):
+    return 2
+
+  @property
+  def use_mini_batch(self):
+    return True
+
+  @property
+  def mini_batch_steps_per_iteration(self):
+    return self.num_points // self.batch_size
+
+
 class KMeansBenchmark(benchmark.Benchmark):
   """Base class for benchmarks."""
 
@@ -405,13 +484,13 @@ class TensorflowKMeansBenchmark(KMeansBenchmark):
       print('Starting tensorflow KMeans: %d' % i)
       tf_kmeans = kmeans_lib.KMeansClustering(
           self.num_clusters,
-          initial_clusters=factorization.KMEANS_PLUS_PLUS_INIT,
+          initial_clusters=kmeans_lib.KMeansClustering.KMEANS_PLUS_PLUS_INIT,
           kmeans_plus_plus_num_retries=int(math.log(self.num_clusters) + 2),
           random_seed=i * 42,
+          relative_tolerance=1e-6,
           config=run_config.RunConfig(tf_random_seed=3))
       tf_kmeans.fit(input_fn=lambda: (constant_op.constant(self.points), None),
-                    steps=50,
-                    relative_tolerance=1e-6)
+                    steps=50)
       _ = tf_kmeans.clusters()
       scores.append(
           tf_kmeans.score(
@@ -439,5 +518,27 @@ class SklearnKMeansBenchmark(KMeansBenchmark):
     self._report(num_iters, start, time.time(), scores)
 
 
+class KMeansTestQueues(test.TestCase):
+
+  def input_fn(self):
+    def _fn():
+      queue = data_flow_ops.FIFOQueue(capacity=10,
+                                      dtypes=dtypes.float32,
+                                      shapes=[10, 3])
+      enqueue_op = queue.enqueue(array_ops.zeros([10, 3], dtype=dtypes.float32))
+      queue_runner.add_queue_runner(queue_runner.QueueRunner(queue,
+                                                             [enqueue_op]))
+      return queue.dequeue(), None
+    return _fn
+
+  # This test makes sure that there are no deadlocks when using a QueueRunner.
+  # Note that since cluster initialization is dependendent on inputs, if input
+  # is generated using a QueueRunner, one has to make sure that these runners
+  # are started before the initialization.
+  def test_queues(self):
+    kmeans = kmeans_lib.KMeansClustering(5)
+    kmeans.fit(input_fn=self.input_fn(), steps=1)
+
+
 if __name__ == '__main__':
   test.main()
