@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-
 """Gradients for operators defined in math_ops.py."""
 from __future__ import absolute_import
 from __future__ import division
@@ -40,8 +39,8 @@ def _SumGrad(op, grad):
   """Gradient for Sum."""
   # Fast path for when reducing to a scalar and ndims is known: adds only
   # Reshape and Tile ops (and possibly a Shape).
-  if (op.inputs[0].get_shape().ndims is not None and op.inputs[1].op.type ==
-      "Const"):
+  if (op.inputs[0].get_shape().ndims is not None and
+      op.inputs[1].op.type == "Const"):
     rank = op.inputs[0].get_shape().ndims
     axes = tensor_util.MakeNdarray(op.inputs[1].op.get_attr("value"))
     if np.array_equal(axes, np.arange(rank)):  # Reduce all dims.
@@ -73,8 +72,7 @@ def _MinOrMaxGrad(op, grad):
   # then the gradient will be divided between them.
   indicators = math_ops.cast(math_ops.equal(y, op.inputs[0]), grad.dtype)
   num_selected = array_ops.reshape(
-      math_ops.reduce_sum(indicators, op.inputs[1]),
-      output_shape_kept_dims)
+      math_ops.reduce_sum(indicators, op.inputs[1]), output_shape_kept_dims)
 
   return [math_ops.div(indicators, num_selected) * grad, None]
 
@@ -96,8 +94,8 @@ def _MeanGrad(op, grad):
   sum_grad = _SumGrad(op, grad)[0]
   input_shape = array_ops.shape(op.inputs[0])
   output_shape = array_ops.shape(op.outputs[0])
-  factor = _safe_shape_div(math_ops.reduce_prod(input_shape),
-                           math_ops.reduce_prod(output_shape))
+  factor = _safe_shape_div(
+      math_ops.reduce_prod(input_shape), math_ops.reduce_prod(output_shape))
   return sum_grad / math_ops.cast(factor, sum_grad.dtype), None
 
 
@@ -127,7 +125,7 @@ def _ProdGrad(op, grad):
     reduced = math_ops.cast(reduction_indices, dtypes.int32)
     idx = math_ops.range(0, array_ops.rank(op.inputs[0]))
     other, _ = array_ops.setdiff1d(idx, reduced)
-    perm = array_ops.concat(0, [reduced, other])
+    perm = array_ops.concat([reduced, other], 0)
     reduced_num = math_ops.reduce_prod(array_ops.gather(input_shape, reduced))
     other_num = math_ops.reduce_prod(array_ops.gather(input_shape, other))
   permuted = array_ops.transpose(op.inputs[0], perm)
@@ -155,9 +153,10 @@ def _SegmentSumGrad(op, grad):
 def _SegmentMeanGrad(op, grad):
   """Gradient for SegmentMean."""
   input_rank = array_ops.rank(op.inputs[0])
-  ones_shape = array_ops.concat(
-      0, [array_ops.shape(op.inputs[1]),
-          array_ops.fill(array_ops.expand_dims(input_rank - 1, 0), 1)])
+  ones_shape = array_ops.concat([
+      array_ops.shape(op.inputs[1]),
+      array_ops.fill(array_ops.expand_dims(input_rank - 1, 0), 1)
+  ], 0)
   ones = array_ops.fill(ones_shape,
                         constant_op.constant(1, dtype=grad.dtype))
   scaled_grad = math_ops.div(grad, math_ops.segment_sum(ones, op.inputs[1]))
@@ -169,61 +168,62 @@ def _SparseSegmentSumGrad(op, grad):
   """Gradient for SparseSegmentSum."""
   input_rows = array_ops.shape(op.inputs[0])[0]
   return (math_ops.unsorted_segment_sum(
-      array_ops.gather(grad, op.inputs[2]),
-      op.inputs[1], input_rows), None, None)
+      array_ops.gather(grad, op.inputs[2]), op.inputs[1], input_rows), None,
+          None)
 
 
 @ops.RegisterGradient("SparseSegmentMean")
 def _SparseSegmentMeanGrad(op, grad):
   """Gradient for SparseSegmentMean."""
   dim0 = array_ops.shape(op.inputs[0])[0]
-  return (math_ops.sparse_segment_mean_grad(grad,
-                                            op.inputs[1],
-                                            op.inputs[2],
-                                            dim0),
-          None, None)
+  return (math_ops.sparse_segment_mean_grad(grad, op.inputs[1], op.inputs[2],
+                                            dim0), None, None)
 
 
 @ops.RegisterGradient("SparseSegmentSqrtN")
 def _SparseSegmentSqrtNGrad(op, grad):
   """Gradient for SparseSegmentSqrtN."""
   dim0 = array_ops.shape(op.inputs[0])[0]
-  return (math_ops.sparse_segment_sqrt_n_grad(grad,
-                                              op.inputs[1],
-                                              op.inputs[2],
-                                              dim0),
-          None, None)
+  return (math_ops.sparse_segment_sqrt_n_grad(grad, op.inputs[1], op.inputs[2],
+                                              dim0), None, None)
 
 
-def _SegmentMinOrMaxGrad(op, grad):
-  """Gradient for SegmentMin and SegmentMax. Both share the same code."""
+def _SegmentMinOrMaxGrad(op, grad, is_sorted):
+  """Gradient for SegmentMin and (unsorted) SegmentMax. They share similar code."""
   zeros = array_ops.zeros(array_ops.shape(op.inputs[0]),
                           dtype=op.inputs[0].dtype)
 
   # Get the number of selected (minimum or maximum) elements in each segment.
   gathered_outputs = array_ops.gather(op.outputs[0], op.inputs[1])
   is_selected = math_ops.equal(op.inputs[0], gathered_outputs)
-  num_selected = math_ops.segment_sum(math_ops.cast(is_selected, grad.dtype),
-                                      op.inputs[1])
+  if is_sorted:
+    num_selected = math_ops.segment_sum(math_ops.cast(is_selected, grad.dtype),
+                                        op.inputs[1])
+  else:
+    num_selected = math_ops.unsorted_segment_sum(math_ops.cast(is_selected, grad.dtype),
+                                                 op.inputs[1], op.inputs[2])
 
   # Compute the gradient for each segment. The gradient for the ith segment is
   # divided evenly among the selected elements in that segment.
   weighted_grads = math_ops.div(grad, num_selected)
   gathered_grads = array_ops.gather(weighted_grads, op.inputs[1])
 
-  return math_ops.select(is_selected, gathered_grads, zeros), None
+  if is_sorted:
+    return array_ops.where(is_selected, gathered_grads, zeros), None
+  else:
+    return array_ops.where(is_selected, gathered_grads, zeros), None, None
 
 
 @ops.RegisterGradient("SegmentMin")
 def _SegmentMinGrad(op, grad):
   """Gradient for SegmentMin."""
-  return _SegmentMinOrMaxGrad(op, grad)
+  return _SegmentMinOrMaxGrad(op, grad, True)
 
 
 @ops.RegisterGradient("SegmentMax")
 def _SegmentMaxGrad(op, grad):
   """Gradient for SegmentMax."""
-  return _SegmentMinOrMaxGrad(op, grad)
+  return _SegmentMinOrMaxGrad(op, grad, True)
 
 
 @ops.RegisterGradient("UnsortedSegmentSum")
@@ -232,6 +232,11 @@ def _UnsortedSegmentSumGrad(op, grad):
   return array_ops.gather(grad, op.inputs[1]), None, None
 
 
+@ops.RegisterGradient("UnsortedSegmentMax")
+def _UnsortedSegmentMaxGrad(op, grad):
+  return _SegmentMinOrMaxGrad(op, grad, False)
+
+
 @ops.RegisterGradient("Abs")
 def _AbsGrad(op, grad):
   x = op.inputs[0]
@@ -336,6 +341,16 @@ def _ExpGrad(op, grad):
     return grad * y
 
 
+@ops.RegisterGradient("Expm1")
+def _Expm1Grad(op, grad):
+  """Returns grad * exp(x)."""
+  x = op.inputs[0]
+  with ops.control_dependencies([grad.op]):
+    x = math_ops.conj(x)
+    y = math_ops.exp(x)
+    return grad * y
+
+
 @ops.RegisterGradient("Log")
 def _LogGrad(op, grad):
   """Returns grad * (1/x)."""
@@ -387,8 +402,8 @@ def _ErfGrad(op, grad):
 def _ErfcGrad(op, grad):
   """Returns -grad * 2/sqrt(pi) * exp(-x**2)."""
   x = op.inputs[0]
-  minus_two_over_root_pi = constant_op.constant(-2 / np.sqrt(np.pi),
-                                                dtype=grad.dtype)
+  minus_two_over_root_pi = constant_op.constant(
+      -2 / np.sqrt(np.pi), dtype=grad.dtype)
   with ops.control_dependencies([grad.op]):
     x = math_ops.conj(x)
     return grad * minus_two_over_root_pi * math_ops.exp(-math_ops.square(x))
@@ -414,7 +429,7 @@ def _DigammaGrad(op, grad):
 
 @ops.RegisterGradient("Igamma")
 def _IgammaGrad(op, grad):
-  """Returns gradient of igamma(a, x) with respect to a and x."""
+  """Returns gradient of igamma(a, x) with respect to x."""
   # TODO(ebrevdo): Perhaps add the derivative w.r.t. a
   a = op.inputs[0]
   x = op.inputs[1]
@@ -424,15 +439,44 @@ def _IgammaGrad(op, grad):
 
   # Perform operations in log space before summing, because Gamma(a)
   # and Gamma'(a) can grow large.
-  partial_x = math_ops.exp(-x + (a-1) * math_ops.log(x) - math_ops.lgamma(a))
+  partial_x = math_ops.exp(-x + (a - 1) * math_ops.log(x) - math_ops.lgamma(a))
+  # TODO(b/36815900): Mark None return values as NotImplemented
   return (None,
           array_ops.reshape(math_ops.reduce_sum(partial_x * grad, rx), sx))
 
 
 @ops.RegisterGradient("Igammac")
 def _IgammacGrad(op, grad):
-  """Returns gradient of igammac(a, x) = 1 - igamma(a, x) w.r.t. a and x."""
-  return [-1 * g if g is not None else None for g in _IgammaGrad(op, grad)]
+  """Returns gradient of igammac(a, x) = 1 - igamma(a, x) w.r.t. x."""
+  _, igamma_grad_x = _IgammaGrad(op, grad)
+  return None, -igamma_grad_x
+
+
+@ops.RegisterGradient("Betainc")
+def _BetaincGrad(op, grad):
+  """Returns gradient of betainc(a, b, x) with respect to x."""
+  # TODO(ebrevdo): Perhaps add the derivative w.r.t. a, b
+  a, b, x = op.inputs
+
+  # two cases: x is a scalar and a/b are same-shaped tensors, or vice
+  # versa; so its sufficient to check against shape(a).
+  sa = array_ops.shape(a)
+  sx = array_ops.shape(x)
+  # pylint: disable=protected-access
+  _, rx = gen_array_ops._broadcast_gradient_args(sa, sx)
+  # pylint: enable=protected-access
+
+  # Perform operations in log space before summing, because terms
+  # can grow large.
+  log_beta = (gen_math_ops.lgamma(a) + gen_math_ops.lgamma(b)
+              - gen_math_ops.lgamma(a + b))
+  partial_x = math_ops.exp(
+      (b - 1) * math_ops.log(1 - x) + (a - 1) * math_ops.log(x) - log_beta)
+
+  # TODO(b/36815900): Mark None return values as NotImplemented
+  return (None,  # da
+          None,  # db
+          array_ops.reshape(math_ops.reduce_sum(partial_x * grad, rx), sx))
 
 
 @ops.RegisterGradient("Zeta")
@@ -450,6 +494,7 @@ def _ZetaGrad(op, grad):
     x = math_ops.conj(x)
     q = math_ops.conj(q)
     partial_q = -x * math_ops.zeta(x + 1, q)
+    # TODO(b/36815900): Mark None return values as NotImplemented
     return (None,
             array_ops.reshape(math_ops.reduce_sum(partial_q * grad, rq), sq))
 
@@ -469,6 +514,7 @@ def _PolygammaGrad(op, grad):
     n = math_ops.conj(n)
     x = math_ops.conj(x)
     partial_x = math_ops.polygamma(n + 1, x)
+    # TODO(b/36815900): Mark None return values as NotImplemented
     return (None,
             array_ops.reshape(math_ops.reduce_sum(partial_x * grad, rx), sx))
 
@@ -537,7 +583,7 @@ def _AsinGrad(op, grad):
     x = math_ops.conj(x)
     x2 = math_ops.square(x)
     one = constant_op.constant(1, dtype=grad.dtype)
-    den = math_ops.sqrt(math_ops.sub(one, x2))
+    den = math_ops.sqrt(math_ops.subtract(one, x2))
     inv = math_ops.reciprocal(den)
     return grad * inv
 
@@ -550,7 +596,7 @@ def _AcosGrad(op, grad):
     x = math_ops.conj(x)
     x2 = math_ops.square(x)
     one = constant_op.constant(1, dtype=grad.dtype)
-    den = math_ops.sqrt(math_ops.sub(one, x2))
+    den = math_ops.sqrt(math_ops.subtract(one, x2))
     inv = math_ops.reciprocal(den)
     return -grad * inv
 
@@ -567,6 +613,16 @@ def _AtanGrad(op, grad):
     return grad * inv
 
 
+@ops.RegisterGradient("Atan2")
+def _Atan2Grad(op, grad):
+  """Returns grad * x / (x^2 + y^2), grad * -y / (x^2 + y^2)."""
+  y = op.inputs[0]
+  x = op.inputs[1]
+  with ops.control_dependencies([grad.op]):
+    grad_inv = grad / (math_ops.square(x) + math_ops.square(y))
+    return x * grad_inv, -y * grad_inv
+
+
 @ops.RegisterGradient("AddN")
 def _AddNGrad(op, grad):
   """Copies the gradient to all inputs."""
@@ -613,16 +669,50 @@ def _MulGrad(op, grad):
 
 @ops.RegisterGradient("Div")
 def _DivGrad(op, grad):
+  """The gradient for the Div operator."""
   x = op.inputs[0]
   y = op.inputs[1]
   sx = array_ops.shape(x)
   sy = array_ops.shape(y)
-  rx, ry = gen_array_ops._broadcast_gradient_args(sx, sy)  # pylint: disable=protected-access
+  # pylint: disable=protected-access
+  rx, ry = gen_array_ops._broadcast_gradient_args(sx, sy)
+  # pylint: enable=protected-access
   x = math_ops.conj(x)
   y = math_ops.conj(y)
-  return (array_ops.reshape(math_ops.reduce_sum(grad / y, rx), sx),
-          array_ops.reshape(math_ops.reduce_sum(grad *
-                                         (-x / math_ops.square(y)), ry), sy))
+  return (array_ops.reshape(math_ops.reduce_sum(math_ops.div(grad, y), rx), sx),
+          array_ops.reshape(
+              math_ops.reduce_sum(grad * math_ops.div(math_ops.div(-x, y), y),
+                                  ry), sy))
+
+
+@ops.RegisterGradient("FloorDiv")
+def _FloorDivGrad(_, unused_grad):
+  """The gradient for the FloorDiv operator."""
+  return None, None
+
+
+@ops.RegisterGradient("TruncateDiv")
+def _TruncateDivGrad(_, unused_grad):
+  return None, None
+
+
+@ops.RegisterGradient("RealDiv")
+def _RealDivGrad(op, grad):
+  """RealDiv op gradient."""
+  x = op.inputs[0]
+  y = op.inputs[1]
+  sx = array_ops.shape(x)
+  sy = array_ops.shape(y)
+  # pylint: disable=protected-access
+  rx, ry = gen_array_ops._broadcast_gradient_args(sx, sy)
+  # pylint: enable=protected-access
+  x = math_ops.conj(x)
+  y = math_ops.conj(y)
+  return (array_ops.reshape(
+      math_ops.reduce_sum(math_ops.realdiv(grad, y), rx),
+      sx), array_ops.reshape(
+          math_ops.reduce_sum(grad * math_ops.realdiv(math_ops.realdiv(-x, y), y),
+                              ry), sy))
 
 
 @ops.RegisterGradient("Pow")
@@ -642,13 +732,12 @@ def _PowGrad(op, grad):
   # Avoid false singularity at x = 0
   if x.dtype.is_complex:
     # real(x) < 0 is fine for the complex case
-    log_x = math_ops.select(
+    log_x = array_ops.where(
         math_ops.not_equal(x, 0), math_ops.log(x), array_ops.zeros_like(x))
   else:
     # There's no sensible real value to return if x < 0, so return 0
-    log_x = math_ops.select(x > 0, math_ops.log(x), array_ops.zeros_like(x))
-  gy = array_ops.reshape(
-      math_ops.reduce_sum(grad * z * log_x, ry), sy)
+    log_x = array_ops.where(x > 0, math_ops.log(x), array_ops.zeros_like(x))
+  gy = array_ops.reshape(math_ops.reduce_sum(grad * z * log_x, ry), sy)
   return gx, gy
 
 
@@ -663,8 +752,8 @@ def _MaximumMinimumGrad(op, grad, selector_op):
   zeros = array_ops.zeros(gradshape, gdtype)
   xmask = selector_op(x, y)
   rx, ry = gen_array_ops._broadcast_gradient_args(sx, sy)
-  xgrad = math_ops.select(xmask, grad, zeros)
-  ygrad = math_ops.select(math_ops.logical_not(xmask), grad, zeros)
+  xgrad = array_ops.where(xmask, grad, zeros)
+  ygrad = array_ops.where(math_ops.logical_not(xmask), grad, zeros)
   gx = array_ops.reshape(math_ops.reduce_sum(xgrad, rx), sx)
   gy = array_ops.reshape(math_ops.reduce_sum(ygrad, ry), sy)
   return (gx, gy)
@@ -707,6 +796,7 @@ ops.NotDifferentiable("LessEqual")
 ops.NotDifferentiable("Greater")
 ops.NotDifferentiable("GreaterEqual")
 ops.NotDifferentiable("Equal")
+ops.NotDifferentiable("ApproximateEqual")
 ops.NotDifferentiable("NotEqual")
 ops.NotDifferentiable("LogicalAnd")
 ops.NotDifferentiable("LogicalOr")
@@ -718,28 +808,31 @@ def _SelectGrad(op, grad):
   c = op.inputs[0]
   x = op.inputs[1]
   zeros = array_ops.zeros_like(x)
-  return (None, math_ops.select(c, grad, zeros),
-          math_ops.select(c, zeros, grad))
+  return (None, array_ops.where(c, grad, zeros),
+          array_ops.where(c, zeros, grad))
 
 
 @ops.RegisterGradient("MatMul")
 def _MatMulGrad(op, grad):
+  """Gradient for MatMul."""
+
   t_a = op.get_attr("transpose_a")
   t_b = op.get_attr("transpose_b")
+  a = math_ops.conj(op.inputs[0])
+  b = math_ops.conj(op.inputs[1])
   if not t_a and not t_b:
-    return (math_ops.matmul(grad, op.inputs[1], transpose_b=True),
-            math_ops.matmul(op.inputs[0], grad, transpose_a=True))
+    grad_a = math_ops.matmul(grad, b, transpose_b=True)
+    grad_b = math_ops.matmul(a, grad, transpose_a=True)
   elif not t_a and t_b:
-    return (math_ops.matmul(grad, op.inputs[1]),
-            math_ops.matmul(grad, op.inputs[0], transpose_a=True))
+    grad_a = math_ops.matmul(grad, b)
+    grad_b = math_ops.matmul(grad, a, transpose_a=True)
   elif t_a and not t_b:
-    return (math_ops.matmul(op.inputs[1], grad, transpose_b=True),
-            math_ops.matmul(op.inputs[0], grad))
+    grad_a = math_ops.matmul(b, grad, transpose_b=True)
+    grad_b = math_ops.matmul(a, grad)
   elif t_a and t_b:
-    return (math_ops.matmul(op.inputs[1], grad, transpose_a=True,
-                            transpose_b=True),
-            math_ops.matmul(grad, op.inputs[0], transpose_a=True,
-                            transpose_b=True))
+    grad_a = math_ops.matmul(b, grad, transpose_a=True, transpose_b=True)
+    grad_b = math_ops.matmul(grad, a, transpose_a=True, transpose_b=True)
+  return grad_a, grad_b
 
 
 @ops.RegisterGradient("SparseMatMul")
@@ -754,8 +847,8 @@ def _SparseMatMulGrad(op, grad):
       # Use heuristic to figure out if grad might be sparse
       grad: (grad.op.type == "ReluGrad")
   }
-  def _SparseMatMul(t1, t2, out_dtype,
-                    transpose_a=False, transpose_b=False):
+
+  def _SparseMatMul(t1, t2, out_dtype, transpose_a=False, transpose_b=False):
     """Helper function to create SparseMatMul op."""
 
     assert t1 in is_sparse and t2 in is_sparse
@@ -764,11 +857,13 @@ def _SparseMatMulGrad(op, grad):
     if transpose_b:
       t2 = array_ops.transpose(t2)
       transpose_b = False
-    prod = math_ops.matmul(t1, t2,
-                           transpose_a=transpose_a,
-                           transpose_b=transpose_b,
-                           a_is_sparse=t1_sparse,
-                           b_is_sparse=t2_sparse)
+    prod = math_ops.matmul(
+        t1,
+        t2,
+        transpose_a=transpose_a,
+        transpose_b=transpose_b,
+        a_is_sparse=t1_sparse,
+        b_is_sparse=t2_sparse)
     if prod.dtype != out_dtype:
       prod = math_ops.cast(prod, out_dtype)
     return prod
@@ -776,19 +871,21 @@ def _SparseMatMulGrad(op, grad):
   dtype_a = op.inputs[0].dtype
   dtype_b = op.inputs[1].dtype
   if not t_a and not t_b:
-    return (_SparseMatMul(grad, op.inputs[1], dtype_a, transpose_b=True),
-            _SparseMatMul(op.inputs[0], grad, dtype_b, transpose_a=True))
+    return (_SparseMatMul(
+        grad, op.inputs[1], dtype_a, transpose_b=True), _SparseMatMul(
+            op.inputs[0], grad, dtype_b, transpose_a=True))
   elif not t_a and t_b:
-    return (_SparseMatMul(grad, op.inputs[1], dtype_a),
-            _SparseMatMul(grad, op.inputs[0], dtype_b, transpose_a=True))
+    return (_SparseMatMul(grad, op.inputs[1], dtype_a), _SparseMatMul(
+        grad, op.inputs[0], dtype_b, transpose_a=True))
   elif t_a and not t_b:
-    return (_SparseMatMul(op.inputs[1], grad, dtype_a, transpose_b=True),
+    return (_SparseMatMul(
+        op.inputs[1], grad, dtype_a, transpose_b=True),
             _SparseMatMul(op.inputs[0], grad, dtype_b))
   elif t_a and t_b:
-    return (_SparseMatMul(op.inputs[1], grad, dtype_a,
-                          transpose_a=True, transpose_b=True),
-            _SparseMatMul(grad, op.inputs[0], dtype_b,
-                          transpose_a=True, transpose_b=True))
+    return (_SparseMatMul(
+        op.inputs[1], grad, dtype_a, transpose_a=True,
+        transpose_b=True), _SparseMatMul(
+            grad, op.inputs[0], dtype_b, transpose_a=True, transpose_b=True))
 
 
 @ops.RegisterGradient("Floor")
@@ -796,6 +893,16 @@ def _FloorGrad(_, unused_grad):
   return [None]
 
 
+@ops.RegisterGradient("Ceil")
+def _CeilGrad(_, unused_grad):
+  return [None]
+
+
+@ops.RegisterGradient("Round")
+def _RoundGrad(_, unused_grad):
+  return [None]
+
+
 @ops.RegisterGradient("Rint")
 def _RintGrad(_, unused_grad):
   # the gradient of Rint is zero
@@ -812,18 +919,18 @@ def _BatchMatMul(op, grad):
 
   if not adj_x:
     if not adj_y:
-      grad_x = math_ops.batch_matmul(grad, y, False, True)
-      grad_y = math_ops.batch_matmul(x, grad, True, False)
+      grad_x = math_ops.matmul(grad, y, adjoint_a=False, adjoint_b=True)
+      grad_y = math_ops.matmul(x, grad, adjoint_a=True, adjoint_b=False)
     else:
-      grad_x = math_ops.batch_matmul(grad, y, False, False)
-      grad_y = math_ops.batch_matmul(grad, x, True, False)
+      grad_x = math_ops.matmul(grad, y, adjoint_a=False, adjoint_b=False)
+      grad_y = math_ops.matmul(grad, x, adjoint_a=True, adjoint_b=False)
   else:
     if not adj_y:
-      grad_x = math_ops.batch_matmul(y, grad, False, True)
-      grad_y = math_ops.batch_matmul(x, grad, False, False)
+      grad_x = math_ops.matmul(y, grad, adjoint_a=False, adjoint_b=True)
+      grad_y = math_ops.matmul(x, grad, adjoint_a=False, adjoint_b=False)
     else:
-      grad_x = math_ops.batch_matmul(y, grad, True, True)
-      grad_y = math_ops.batch_matmul(grad, x, True, True)
+      grad_x = math_ops.matmul(y, grad, adjoint_a=True, adjoint_b=True)
+      grad_y = math_ops.matmul(grad, x, adjoint_a=True, adjoint_b=True)
 
   return grad_x, grad_y
 
@@ -875,8 +982,10 @@ def _ComplexAbsGrad(op, grad):
 
 @ops.RegisterGradient("Cast")
 def _CastGrad(op, grad):
-  t = [dtypes.float16, dtypes.float32, dtypes.float64,
-       dtypes.bfloat16, dtypes.complex64, dtypes.complex128]
+  t = [
+      dtypes.float16, dtypes.float32, dtypes.float64, dtypes.bfloat16,
+      dtypes.complex64, dtypes.complex128
+  ]
   src_type = op.inputs[0].dtype.base_dtype
   dst_type = grad.dtype.base_dtype
   if src_type in t and dst_type in t:
@@ -885,48 +994,6 @@ def _CastGrad(op, grad):
     return None
 
 
-def _FFTSizeForGrad(grad, rank):
-  return math_ops.reduce_prod(
-      array_ops.slice(
-          array_ops.reverse(array_ops.shape(grad), (True,)), (0,), (rank,)))
-
-
-@ops.RegisterGradient("FFT")
-def _FFTGrad(_, grad):
-  size = math_ops.cast(_FFTSizeForGrad(grad, 1), dtypes.float32)
-  return math_ops.ifft(grad) * math_ops.complex(size, 0.)
-
-
-@ops.RegisterGradient("IFFT")
-def _IFFTGrad(_, grad):
-  rsize = 1. / math_ops.cast(_FFTSizeForGrad(grad, 1), dtypes.float32)
-  return math_ops.fft(grad) * math_ops.complex(rsize, 0.)
-
-
-@ops.RegisterGradient("FFT2D")
-def _FFT2DGrad(_, grad):
-  size = math_ops.cast(_FFTSizeForGrad(grad, 2), dtypes.float32)
-  return math_ops.ifft2d(grad) * math_ops.complex(size, 0.)
-
-
-@ops.RegisterGradient("IFFT2D")
-def _IFFT2DGrad(_, grad):
-  rsize = 1. / math_ops.cast(_FFTSizeForGrad(grad, 2), dtypes.float32)
-  return math_ops.fft2d(grad) * math_ops.complex(rsize, 0.)
-
-
-@ops.RegisterGradient("FFT3D")
-def _FFT3DGrad(_, grad):
-  size = math_ops.cast(_FFTSizeForGrad(grad, 3), dtypes.float32)
-  return math_ops.ifft3d(grad) * math_ops.complex(size, 0.)
-
-
-@ops.RegisterGradient("IFFT3D")
-def _IFFT3DGrad(_, grad):
-  rsize = 1. / math_ops.cast(_FFTSizeForGrad(grad, 3), dtypes.float32)
-  return math_ops.fft3d(grad) * math_ops.complex(rsize, 0.)
-
-
 @ops.RegisterGradient("Cross")
 def _CrossGrad(op, grad):
   u = op.inputs[0]
@@ -939,8 +1006,10 @@ def _CumsumGrad(op, grad):
   axis = op.inputs[1]
   exclusive = op.get_attr("exclusive")
   reverse = op.get_attr("reverse")
-  return [math_ops.cumsum(grad, axis, exclusive=exclusive,
-                          reverse=not reverse), None]
+  return [
+      math_ops.cumsum(
+          grad, axis, exclusive=exclusive, reverse=not reverse), None
+  ]
 
 
 @ops.RegisterGradient("Cumprod")
@@ -952,6 +1021,6 @@ def _CumprodGrad(op, grad):
 
   # TODO This fails when x contains 0 and should be fixed
   prod = math_ops.cumprod(x, axis, exclusive=exclusive, reverse=reverse)
-  out = math_ops.cumsum(prod * grad, axis, exclusive=exclusive,
-                        reverse=not reverse)
+  out = math_ops.cumsum(
+      prod * grad, axis, exclusive=exclusive, reverse=not reverse)
   return [out / x, None]
