@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-"""Python layer for sets.
+"""Tensorflow set operations.
 
 @@set_size
 @@set_intersection
